//---------------------------------------------------------------------------
#include <stdlib.h>
#include <stdio.h>
#include <windows.h>
#include <math.h>
#include <vcl.h>



#pragma hdrstop
#include "simple_scope.h"
#include "cscope interface.c"





//**************************************************************************************
//
//
//  See the timer_tick function at the end of this file for where all the action happens
//
//**********************************************************************************

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"

#define num_points      1000
#define MAX_PATHNAME_LEN        200

void InitializeGUI (void);

//int 	panel_handle;
float64 	a_vert = 1.0, b_vert = 1.0, horz = 0.001;
float64	sig_gen_volts = 1.0, sig_gen_freq = 1000.0;
int32	channel = 1;
int32 	trig_channel = 0;
float64	trig_volts = 0.0;
int32	single_acquire = false;
int32	auto_acquire = false;
float32  a_waveform[max_samples], b_waveform[max_samples];
float64	dT, T0;
char 	pathname[MAX_PATHNAME_LEN];
int32	waiting_for_trigger;	//0 means we have not started an acquire. 1 means we are waiting for the trigger
uInt16	trigger_action;	//0 = auto, 1 = single, 3 = stop
int32	Interface_Source = 0;
uInt32	Interface_address = 0xc0a80166;    //192.168.1.102
int32	Cscope_index;
int32	cscope_status;
int32	function_value;		//contains function number to send

int32	num_samples;
int32	vals_changed;
uInt32 		digital_pattern =0x00010001;	//digital pattern used for digital triggering
int32	f_timer_active;	 //true if timer may become active

//****************************************************************************
//Support routines

float64 v_div[9] = { 5,2,1,0.5,0.2,0.1,0.05,0.02,0.01};
float64 t_div[25] = { 1,0.5,0.2,0.1,0.05,0.02,0.01,0.005,0.002,0.001,
					  500e-6,200e-6,100e-6, 50e-6,20e-6,10e-6,5e-6,2e-6,1e-6,
					  500e-9,200e-9,100e-9,50e-9,20e-9,10e-9};
float64 freqs[21] = {1.0,2,5,10,20,50,100,200,500,1000,2000,5000,10000,
					 20000,50000,100000,200000,500000,1000000,2000000,5000000};

//-------------------------------------------------------------------------
#define PREFIX_START (-24)
/* Smallest power of then for which there is a prefix defined.
   If the set of prefixes will be extended, change this constant
   and update the table "prefix". */

char *eng(float64 value, int32 digits, int32 numeric)
{
  static char *prefix[] = {
  "y", "z", "a", "f", "p", "n", "u", "m", "",
  "k", "M", "G", "T", "P", "E", "Z", "Y"
  };
#define PREFIX_END (PREFIX_START+\
(int32)((sizeof(prefix)/sizeof(char *)-1)*3))

      int32 expof10;
      static unsigned char result[100];
      unsigned char *res = result;

      if (value < 0.)
        {
            *res++ = '-';
            value = -value;
        }
		if (value != 0.0)
			{
			expof10 = (int32) log10(value);
		   if(expof10 > 0)
			 expof10 = (expof10/3)*3;
		   else
			 expof10 = (-expof10+3)/3*(-3);

		   value *= pow(10,-expof10);

		   if (value >= 1000.)
			  { value /= 1000.0; expof10 += 3; }
		   else if(value >= 100.0)
			  digits -= 2;
		   else if(value >= 10.0)
			  digits -= 1;
		   }
	   else
			{
			expof10 = 0;	//no prefix
			digits=3;
			}
      if(numeric || (expof10 < PREFIX_START) ||
                    (expof10 > PREFIX_END))
        sprintf(res, "%.*fe%d", digits-1, value, expof10);
      else
        sprintf(res, "%.*f %s", digits-1, value,
          prefix[(expof10-PREFIX_START)/3]);
      return result;
}
//----------------------------------------------------------------------
uInt32 string_to_IP(AnsiString ip_string)
//converts the string pointed at by ip_str into an IP address,
//assuming the format nnn.nnn.nnn.nnn
//
{
uInt32	ip_adr;
char	* pch;

char* ip_str = new char[ ip_string.Length() + 1 ];

strcpy( ip_str, ip_string.c_str() );
ip_adr = 0;
pch = strtok( ip_str,".");
while (pch != NULL)
	{
	ip_adr = (ip_adr<<8) + atoi(pch);
	pch = strtok (NULL, ".");
	}
return ip_adr;
}

//------------------------------------------------------------------------

void Setval(TLabel* ctrl, float64 value, char *suffix)
{
	  CHAR ctrl_val[10];
	  sprintf(ctrl_val, "%s%s", eng(value, 3, 0),suffix);
	  ctrl->Caption = ctrl_val;
}

void Sethex(TLabel* ctrl, uInt32 value, char *suffix)
{
	  CHAR ctrl_val[10];
	  sprintf(ctrl_val, "%x%s", value ,suffix);
	  ctrl->Caption = ctrl_val;
}

void SetInt(TLabel* ctrl, uInt32 value, char *suffix)
{
	  CHAR ctrl_val[10];
	  sprintf(ctrl_val, "%d%s", value ,suffix);
	  ctrl->Caption = ctrl_val;
}

void SetReal(TLabel* ctrl, float64 value, char *suffix)
{
	  CHAR ctrl_val[10];
	  sprintf(ctrl_val, "%g%s", value ,suffix);
	  ctrl->Caption = ctrl_val;
}
//***************************************************************************
TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------


void InitializeGUI ()
{
single_acquire = false;
auto_acquire = false;
waiting_for_trigger = false;
vals_changed = true;
trigger_action = acq_stop;   //stop triggering
TeeDefaultCapacity = 8000;     //allocate enough room for 8000 samples
Setval(Form1->Adiv, a_vert,"V");
Setval(Form1->Bdiv, b_vert,"V");
Setval(Form1->Tdiv, horz,"s");
Setval(Form1->TrigV, trig_volts,"V");
Setval(Form1->SigGenF, sig_gen_freq,"Hz");
Setval(Form1->SigGenV, sig_gen_volts,"V");
Form1->SetA_Vertical-> Position = 2;
Form1->Chart1->LeftAxis->Minimum = -a_vert*v_divisions/2;
Form1->Chart1->LeftAxis->Maximum = a_vert*v_divisions/2 ;
Form1->Chart1->LeftAxis->Increment = a_vert;
Form1->SetB_Vertical-> Position = 2;
Form1->Chart1->RightAxis->Minimum = -a_vert*v_divisions/2;
Form1->Chart1->RightAxis->Maximum = a_vert*v_divisions/2 ;
Form1->Chart1->RightAxis->Increment = b_vert;
Form1->SetHorizontal-> Position = 9;
Form1->Chart1->BottomAxis->Minimum = -horz*t_divisions/2;
Form1->Chart1->BottomAxis->Maximum = horz*t_divisions/2 ;
Form1->Chart1->BottomAxis->Increment = horz;
Form1->Timer1 ->Enabled = true;
f_timer_active  = true;
Form1->Timer1->Enabled = true;
}
//**************************************************************************
void __fastcall TForm1::FormClose(TObject *Sender, TCloseAction &Action)
{
f_timer_active = false;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::QuitButtonClick(TObject *Sender)
{
f_timer_active = false;
}


//---------------------------------------------------------------------------
void __fastcall TForm1::SetA_VerticalChange(TObject *Sender)
{
a_vert =  v_div[SetA_Vertical-> Position];
Setval(Form1->Adiv, a_vert,"V");
vals_changed = true;
Chart1->LeftAxis->Minimum = -a_vert*v_divisions/2;
Chart1->LeftAxis->Maximum = a_vert*v_divisions/2 ;
Chart1->LeftAxis->Increment = a_vert;

}
//---------------------------------------------------------------------------

void __fastcall TForm1::SetHorizontalChange(TObject *Sender)
{
horz =  t_div[SetHorizontal-> Position];
Setval(Form1->Tdiv, horz,"s");
vals_changed = true;
Chart1->BottomAxis->Minimum = -horz*t_divisions/2;
Chart1->BottomAxis->Maximum = horz*t_divisions/2 ;
Chart1->BottomAxis->Increment = horz;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::SetB_VerticalChange(TObject *Sender)
{
b_vert =  v_div[SetB_Vertical-> Position];
Setval(Form1->Bdiv, b_vert,"V");
vals_changed = true;
Chart1->RightAxis->Minimum = -b_vert*v_divisions/2;
Chart1->RightAxis->Maximum = b_vert*v_divisions/2 ;
Chart1->RightAxis->Increment = b_vert;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Set_SigGen_FreqChange(TObject *Sender)
{
sig_gen_freq =  freqs[Set_SigGen_Freq-> Position];
Setval(Form1->SigGenF, sig_gen_freq, "Hz");
vals_changed = true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Set_SigGen_VoltsChange(TObject *Sender)
{
sig_gen_volts = Set_SigGen_Volts-> Position * 0.2;
Setval(Form1->SigGenV, sig_gen_volts, "V");
vals_changed = true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Set_Trigger_VoltsChange(TObject *Sender)
{
trig_volts = Set_Trigger_Volts-> Position * -0.2;
Setval(Form1->TrigV, trig_volts, "V");
vals_changed = true;
}
//---------------------------------------------------------------------------



void __fastcall TForm1::FormActivate(TObject *Sender)
{
int32		nErr;
char	message[100];

	 nErr =scope_init ();
	 if (nErr>0)
		{
		 MessageDlg("There was a problem with the Cleverscope DLL, the error code = " + IntToStr(nErr),
				mtInformation, TMsgDlgButtons() << mbOK, 0);

		}


	 InitializeGUI();

}
//---------------------------------------------------------------------------


void __fastcall TForm1::SingleAcquireClick(TObject *Sender)
{
vals_changed = true;
Trig_LED -> Brush -> Color = clWhite;
auto_acquire = false;
waiting_for_trigger = false;
TriggeredAcquire->Caption="Auto";

if (single_acquire)
	{
	single_acquire = false;
	SingleAcquire->Caption="Single";
	trigger_action = acq_stop;   //stop triggering
	}
else
	{
	single_acquire= true;
	SingleAcquire->Caption="Wait...";
	trigger_action = acq_single;	//use triggered capture
	}
}
//---------------------------------------------------------------------------


void __fastcall TForm1::TriggeredAcquireClick(TObject *Sender)
{
vals_changed = true;
single_acquire = false;
Trig_LED -> Brush -> Color = clWhite;
waiting_for_trigger = false;
SingleAcquire->Caption="Single";
if (auto_acquire)
	{
	auto_acquire = false;
	TriggeredAcquire->Caption="Auto";
	trigger_action = acq_stop;   //stop triggering
	}
else
	{
	auto_acquire = true;
	TriggeredAcquire->Caption="Active...";
	trigger_action = acq_auto;	//use auto capture
	}
}
//---------------------------------------------------------------------------
//This is the main event function. It checks to see if any values have changed, and if
//they have, it updates the acquire variable.
//Next it sees if an acquisition is to be made - and if so, makes it.
//Next it checks if there are results - if so, get them
//Finally it checks if the changes that should have been updated by teh acquire (if it happened)
//have happened, and if not, it updates the acquisition unit itself.

void __fastcall TForm1::Timer1Timer(TObject *Sender)
{

int32	i;
float64	xval;
uInt16	status;

Timer1->Enabled = false;			//we stop timer events, in case this process takes longer
									//than a timer tick
status = get_CAU_status();

if (cscope_status != status)
	{
	cscope_status = status;
	Cscope_Status->ItemIndex = cscope_status;
	}

if (vals_changed)
    {
	 update_values(a_vert, b_vert, horz, num_points,
	 sig_gen_freq,sig_gen_volts,trig_volts, trig_channel,trigger_action,digital_pattern );
	 vals_changed = false;
	 }

if (waiting_for_trigger)
	{
	if  (check_for_samples())
		{
		Trig_LED -> Brush -> Color = clRed;
		waiting_for_trigger = false;
		if (single_acquire)  SingleAcquire->Caption="Single";
		single_acquire = false;
		scope_read_waveform (a_waveform, b_waveform, &num_samples, &dT, &T0);
		xval = T0 ;
		Series1->Clear();
		Series2->Clear();
		for (i=0; i < num_samples; i++)
			{
			Series1->AddXY(xval,a_waveform[i]);
			Series2->AddXY(xval,b_waveform[i]);
			xval = xval+dT;
			}
		}
	else
		{
		Trig_LED -> Brush -> Color = clWhite;
		}
	}

if ((!waiting_for_trigger) && (single_acquire || auto_acquire))
	 {

	 waiting_for_trigger = true;
	 scope_acquire ();
	 }
if (vals_changed == true)
	  {
	  update_values(a_vert, b_vert, horz, num_points,
	  sig_gen_freq,sig_gen_volts,trig_volts, trig_channel,trigger_action, digital_pattern);
	  scope_config();
	  vals_changed = false;		//updated if acquire did not update them
	  }
Timer1->Enabled = f_timer_active;
if (!f_timer_active)            //we do it here at a nice quantized point
	{
	scope_close ();
	Close();
	}

}
//---------------------------------------------------------------------------

void __fastcall TForm1::Intf_sourceChange(TObject *Sender)
{
		  Interface_Source = Intf_source->ItemIndex;
		cscope_intf(Cscope_index, Interface_Source, Interface_address);

}
//---------------------------------------------------------------------------

void __fastcall TForm1::IP_addressChange(TObject *Sender)
{


		 Interface_address =  string_to_IP(IP_address->Text);
		cscope_intf(Cscope_index, Interface_Source, Interface_address);

}
//---------------------------------------------------------------------------

void __fastcall TForm1::Cscope_unit_indexChange(TObject *Sender)
{
		Cscope_index = Cscope_unit_index->ItemIndex;
		cscope_intf(Cscope_index, Interface_Source, Interface_address);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::RestartClick(TObject *Sender)
{
		 cscope_intf(Cscope_index, Interface_Source, Interface_address);
}
//---------------------------------------------------------------------------



void __fastcall TForm1::Trigger_channelChange(TObject *Sender)
{
		trig_channel = Trigger_channel->ItemIndex;
		vals_changed = true;
}
//---------------------------------------------------------------------------


void __fastcall TForm1::Cmd_FunctionChange(TObject *Sender)
{
		function_value = Cmd_Function->ItemIndex;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Cmd_getClick(TObject *Sender)
{
	   SetReal(Cmd_value, get_function(function_value,0), " ");
}
//---------------------------------------------------------------------------

