#include <ansi_c.h>
 #include "Cscope Control Driver.h"
 #include "cscope interface.h"




int scope_err;

float32		a_samples[max_samples];	  	//contains the a channel samples
float32		b_samples[max_samples];	  	//contains the b channel samples
uInt16	dig_samples[max_samples];	//contains teh digotal channel values

int32		samples_required;			//the number of samples to capture and display
int32		samples_returned;			//actual number of samples returned
float64		dt,t0;						//holds time increment and start value
int32		frames_returned;			//number of frames returned after sequential transfer
static	   	TD1	acquire;				//holds the acquire definition
static		TD2	error;					//holds the error value
static		LVBoolean	got_samples;	//set to 1 when we have samples
int32 		frame_number;				//frame number to extract for multi-frame replay
uInt16	 CAU_status;			//gives status of CAU. 0 = Runtime Closed, 1 = Closed ,
										//2 = Open, 3 = Fault and open, 4 = Fault and Closed
unsigned char tx_chars[max_chars];		//contains character array to send to link
unsigned char rx_chars[max_chars];		//contains character array received from link
float64		function_result;			//result of function call

int32		acquisition_unit;			//Acquisition unit we are talking to, 0..7

//**************************************************
uInt16 call_cscope_control_driver(uInt16 command)
//use this routine to call the control driver with a particular command
{
CscopeControlDriver(acquisition_unit, command, acquire.StartTime, acquire.StopTime, samples_required,frame_number, &acquire, &got_samples,
                    &t0, &dt, &samples_returned, &frames_returned, a_samples, sizeof(a_samples),  b_samples, sizeof(b_samples),
                    dig_samples, sizeof(dig_samples), &CAU_status, &error);


return CAU_status;
}
//**************************************************************

float64 get_function(uInt16 cmd_value, float64 cmd_parameter)
//use this routine to call the control driver function command

{
CscopeFunction(acquisition_unit, cmd_value, cmd_parameter,
	tx_chars, sizeof(tx_chars), &function_result, rx_chars, sizeof(rx_chars),
	&error);

return function_result;
}

//***********************************************************************

int32	scope_init (void)
//Assumes the scope is connected and opens it.
//Sets up the default values

{
acquire.AcquireMode = 3;		//automatic  (0 = single, 1 = automatic, 2 = triggered, 3 = stop)
acquire.AcquisitionMode = 1;	//peak captured
acquire.Acquirer =  4;			//cleverscope is the acquirer
acquire.TransferChans = 2;		//transfer both channels
acquire.AMaxScale = 2;			// Volts range = +/-2
acquire.AMinScale = -2;
acquire.BMaxScale = 2;
acquire.BMinScale = -2;
acquire.AProbe = 0;				//x1
acquire.BProbe = 0;				//x1
acquire.ACoupling = 1;			//DC
acquire.BCoupling = 1;			//DC
acquire.ABandwidth = 0;			//This is filter now
acquire.BBandwidth = 0;			//100 MHz
acquire.TriggerSource = 0;		//A Chan trigger
acquire.TriggerAmplitude = 0;	//Trigger at zero volts
acquire.ATriggerAmplitude = 0;
acquire.BTriggerAmplitude = 0;
acquire.TriggerFilter = 0;		//No trigger filter
acquire.TrigSlope = 0;			//rising
acquire.TriggerHoldoff = 0;
acquire.DigPatternRqd = 0;		//not used
acquire.DigPattern =  0;		//not used
acquire.ExtTrigThreshold = 0;
acquire.DigInputThreshold = 2;
acquire.StartTime =  -0.005;	//-5 msecs
acquire.StopTime =   0.005;		//5 msecs
acquire.SampleInterval = 10e-9;
acquire.Port = 0;
acquire.NumDivisions = 10;
acquire.NumSeqFrames = 1;		 //1 frame per capture
acquire.NumBuffers = 2;			 //2 frames miniumum in acquisition unit
acquire.SigGenFreq = 1000;		//1kHz output
acquire.SigGenAmp = 1;			//1V amplitude
acquire.SigGenOffset = 0;
acquire.SigGenWaveform = 0;		//sine
acquire.SigGenSweep = 0;
acquire.SigGenFunc = 0;
acquire.SigGenFreq2 = 0;
acquire.SigGenPhase = 0;
acquire.Trig2Function = 0;		//not used
acquire.MinTriggerPeriod = 0.0000001;
acquire.MaxTriggerPeriod = 1;
acquire.TriggerCount = 1;
acquire.Trig2Slope = 0;
acquire.Trig2SourceChan = 0;
acquire.Trig2Level = 0;
acquire.DigPattern2Rqd = 0;
acquire.DigPattern2 = 0;
acquire.Trigger2Source = 0;
acquire.WaveformAverages = 1;
acquire.ValueChanged = 1;
acquire.SamplerResolution = 0;	//0 = 10 bit (1 = 12 bit, 2 = 14 bit).

acquire.ExtSampleClock = 0;
acquire.IntfSource = 0;			//0 = USB, 1 = Ethernet - open given IP address,
								//2 = Ethernet - open next free CAU, 3 = Ethernet - open  given Serial number
acquire.TCPPort = 53270;
acquire.UpdateRate =0;
acquire.TransferSize =0;
samples_required = 1000;
frame_number = 0;
acquisition_unit = 0;

return call_cscope_control_driver(c_init);
}

//**************************************************
int32	cscope_interface(int32 unit_index, uInt16 Intf_source, uInt32 TCP_adr)
//Updates the interface spec. If the CAU_statrus indicates the scope is not open, attempts to open it.
//Call this after changing the IP address etc.
{
acquire.ValueChanged++;
acquire.IntfSource = Intf_source;
acquire.TCPAdr = TCP_adr;
acquisition_unit = unit_index;
CAU_status = call_cscope_control_driver(c_idle); 
if (CAU_status != c_open)
	{
	return call_cscope_control_driver(c_init);
	}
else
	{
	return 0;
	}
}

//*************************************************
int32 scope_close (void)
//closes the scope
{
if  (CAU_status != c_runtime_closed) 
	{
   call_cscope_control_driver(c_close);    //closes active unit
	call_cscope_control_driver(c_finish);    //closes all units
   }
return CAU_status;

}


//****************************************************
void update_values(float64 a_div, float64 b_div, float64 t_div, int32 number_of_points,
					float64 freq, float64 sigvolts, float64 trigvolts, int32 trig_chan,
					uInt16 trigger_action, uInt32 dig_pattern)
//updates the acquire variable only. If triggered = 1 then we do triggered capture, otherwise auto capture
{
acquire.AMaxScale = v_divisions * a_div / 2;
acquire.AMinScale = - acquire.AMaxScale;
acquire.BMaxScale = v_divisions * b_div / 2;
acquire.BMinScale = - acquire.BMaxScale;
acquire.StopTime = t_divisions * t_div /2;
acquire.StartTime = - acquire.StopTime;
acquire.SigGenFreq = freq;
acquire.SigGenAmp = sigvolts;


samples_required = number_of_points;
acquire.AcquireMode = trigger_action;
acquire.TriggerSource = trig_chan;
switch (trig_chan)
    {
    case trig_chan_a:
    	acquire.TriggerAmplitude = trigvolts;
    	break;
    case trig_chan_b:
    	acquire.TriggerAmplitude = trigvolts;
    	break;
   	case trig_ext:
   		acquire.ExtTrigThreshold = trigvolts;
   		break;
   	case trig_dig:
   		acquire.DigInputThreshold = trigvolts;
   		break;
   	}
acquire.DigPattern = dig_pattern;

}

//*******************************************************
int32 scope_config (void)
//Configures major values for the acquisition unit.
{
return call_cscope_control_driver(c_update);
}

//*************************************************************************

int32	scope_acquire (void)
//start an acquisition

{
return call_cscope_control_driver(c_acquire);
}

//**************************************************************************
int32 scope_read_waveform (float32 a_waveform[max_samples], float32 b_waveform[max_samples], int32 *num_samples, float64 *delta_t, float64 *t_zero)
//returns the last read waveform for the given channel
{
int i;

for (i=0; i<samples_returned; i++)
	{
	a_waveform[i] = a_samples[i];
	}
for (i=0; i<samples_returned; i++)
	{
	b_waveform[i] = b_samples[i];
	}

*delta_t = dt;
*t_zero = t0;
*num_samples = samples_returned;
return scope_err;
}

//**************************************************************************

int32 check_for_samples(void)
//checks to see if samples have been returned. If so returns 1, else 0
{
call_cscope_control_driver(c_check);
return got_samples;
}

//****************************************************************************

int32	get_CAU_status(void)
//gets the current CAU status
{
call_cscope_control_driver(c_idle);
return CAU_status;
}

//**********************************************************************************

uInt32 string_to_IP(int8 * ip_str)
//converts the string pointed at by ip_str into an IP address,
//assuming the format nnn.nnn.nnn.nnn
//
{
uInt32	ip_adr;
int8	* pch;

ip_adr = 0;
pch = strtok( ip_str,".");
while (pch != NULL)
	{
	ip_adr = (ip_adr<<8) + atoi(pch);
	pch = strtok (NULL, ".");
	}
return ip_adr;
}

//***********************************************************************************
