unit cscope_interface;

interface

uses     Cscope_control_driver;

const    max_samples =  10000;
			t_divisions=10;
			v_divisions=8;
        max_chars =  10000;
           
type    signal = array[0..max_samples-1] of single;

procedure scope_init(var status:integer);
         {initializes cleverscope dll. Returns current status}

procedure cscope_intf(unit_index, Intf_source:integer; TCP_adr:longword);
			{sets the current cleverscope interface}

procedure scope_close(var status:integer);
         {closes cleverscope dll. Returns 0 if all ok, otherwise error code}

procedure scope_config (var status:integer);
         {configures the acquisition unit, using the current acquire value}


procedure	scope_acquire (var status:integer);
           {starts an acquisition. returns error code, = 0 if ok}

procedure scope_read_waveform (var a_waveform, b_waveform : signal;
                 var num_samples:integer; var delta_t, t_zero : double;
                 var status:integer);
          {reads the samples from the buffer, including waveform details}

function check_for_samples:boolean;
         {returns true if we have got the samples}

procedure update_values(a_div, b_div, t_div :double ; number_of_points:integer;
			freq,sigvolts,trigvolts:double; trig_chan:integer; trigger_action:word;
           dig_pattern:longword);
           {updates the values in the acquire structure but does not send them
            to the scope - you need to run either Scope_acquire or scope_config
            to do that. trigger_action = single (0), auto (1), or stop (3)
            Trig_chan = 0 for Chan A, 1 = chan B}

function get_CAU_status:integer;
			//gets the current CAU status

procedure get_function(cmd_value:integer; cmd_parameter:double; var function_result, result_a, result_b: double);
//gets the function value

//Utility functions:
function string_to_IP(IP_address: string):longword;
	//converts string form of IP address to word.

type 	scope_acquisition = (acq_single, acq_auto, acq_triggered, acq_stop);
		commands = (c_init, c_acquire, c_replay, c_check, c_update, c_close, c_idle, c_function, c_get_frame, c_finish);
		scope_status = (c_runtime_closed, c_closed, c_open,  c_fault, c_fault_closed);
		scope_trigger_channel = (trig_chan_a, trig_chan_b, trig_ext, trig_dig);
		scope_interface = (if_usb, if_ethernet_find_ip, if_ethernet_first_ip, if_ethernet_find_sernum);


//------------------------------------------------------------------
implementation



var
a_samples, b_samples:	array[0..max_samples-1] of single;   //contains the a, b channel samples
dig_samples:			array[0..max_samples-1] of word;	//contains teh digotal channel values
samples_required:		integer;			//the number of samples to capture and display
samples_returned:		integer;			//actual number of samples returned
dt,t0:					double;				//holds time increment and start value
frames_returned:		integer;			//number of frames returned after sequential transfer
acquire: 				Acquire_record;	   //holds the acquire definition
errorOut: 				Error_record;				//holds the error value
got_samples: 			byte;               //set to 1 when we have samples
frame_number:			integer;			//the frame to get, if a multi-frame return done.
CAU_status:			word;			//gives status of CAU. 0 = Runtime Closed, 1 = Closed ,
										//2 = Open, 3 = Fault and open, 4 = Fault and Closed
acquisition_unit:		integer;			//Acquisition unit we are talking to, 0..7

rx_chars:				array[0..max_chars-1] of byte;
tx_chars:				array[0..max_chars-1] of byte;
function_result:		double;
result_a:               double;
result_b:               double;

//**************************************************************
procedure call_control_driver(command: commands);
//works totally with local globals to make it easy

begin
CscopeControlDriver(acquisition_unit, word(command),acquire.StartTime,acquire.StopTime,
	samples_required,frame_number,@Acquire,@got_samples,@t0,@dT,
   @samples_returned,@frames_returned,
   @a_samples[0],sizeof(a_samples),@b_samples[0],sizeof(b_samples),
	@dig_samples[0],sizeof(dig_samples),@CAU_status, @errorOut) ;
end;

//**************************************************************

procedure get_function(cmd_value:integer; cmd_parameter: double; var function_result, result_a, result_b : double);
//again works total with local globals

begin
CscopeFunction(acquisition_unit, word(cmd_value), cmd_parameter, @tx_chars[0], sizeof(tx_chars),
					@function_result, @result_a, @result_b, @rx_chars[0], sizeof(rx_chars), @errorOut);
end;

//*******************************************************

procedure scope_init(var status:integer);
//Assumes the scope is connected and opens it. 
//Sets up the default values

begin
acquire.AcquireMode := 1;    //0 single,  1 automatic
acquire.AcquisitionMode := 1;    //peak captured
acquire.Acquirer :=  4;    //cleverscope is the acquirer
acquire.TransferChans := 1;    //transfer both channels
acquire.AMaxScale := 5;    // Volts range := +/-2
acquire.AMinScale := -5;
acquire.BMaxScale := 5;
acquire.BMinScale := -5;
acquire.AProbe := 0;    //x1
acquire.BProbe := 0;    //x1
acquire.ACoupling := 1;    //DC
acquire.BCoupling := 1;    //DC
acquire.ABandwidth := 1;    //100 MHz
acquire.BBandwidth := 1;    //100 MHz
acquire.TriggerSource := 0;    //A Chan trigger
acquire.TriggerAmplitude := 0;    //Trigger at zero volts
acquire.ATriggerAmplitude := 0;
acquire.BTriggerAmplitude := 0;
acquire.TriggerFilter := 0;    //No trigger filter
acquire.TrigSlope := 0;    //rising
acquire.TriggerHoldoff := 0;
acquire.DigPatternRqd := 0;    //not used
acquire.DigPattern :=  0;    //not used
acquire.ExtTrigThreshold := 0;
acquire.DigInputThreshold := 2;
acquire.StartTime :=  -0.005;  //-5 msecs
acquire.StopTime :=   0.005;    //5 msecs
acquire.PreTrigTime := 0.005;
acquire.Port := 0;
acquire.NumDivisions := 10;
acquire.NumSeqFrames := 1;
acquire.NumBuffers := 2;
acquire.SigGenFreq := 1000;    //1kHz output
acquire.SigGenAmp := 2;    //2V amplitude
acquire.SigGenOffset := 0;
acquire.SigGenWaveform := 0;    //sine
acquire.SigGenSweep := 0;
acquire.SigGenFunc := 0;
acquire.SigGenFreq2 := 0;
acquire.SigGenPhase := 0;
acquire.Trig2Function := 0;    //not used
acquire.MinTriggerPeriod := 0.0000001;
acquire.MaxTriggerPeriod := 1;
acquire.TriggerCount := 1;
acquire.Trig2Slope := 0;
acquire.Trig2SourceChan := 0;
acquire.Trig2Level := 0;
acquire.DigPattern2Rqd := 0;
acquire.DigPattern2 := 0;
acquire.Trigger2Source := 0;
acquire.WaveformAverages := 1;
acquire.ValueChanged := 1;
acquire.SamplerResolution := 0; //0 := 10 bit (1 := 12 bit, 2 := 14 bit).
acquire.TransferSize :=0;   //0= normal
samples_required:=1000;
acquire.IntfSource := 0;			//0 = USB, 1 = Ethernet - open given IP address,
								//2 = Ethernet - open next free CAU, 3 = Ethernet - open  given Serial number
acquire.TCPPort := 53270;
acquire.TCPAdr := $C0A80169;	//192.168.1.105
acquire.UnitsAreLinked := 0;
acquire.ExtSampleClock := 0;
acquire.UpdateRate :=0;
acquire.TransferSize :=0;

frame_number:=0;
call_control_driver(c_init);
  //initialize the cleverscope acquisition unit
status:=  CAU_status;
end;

//**************************************************
procedure cscope_intf(unit_index, Intf_source:integer; TCP_adr:longword);
//Updates the interface spec. If the CAU_statrus indicates the scope is not open, attempts to open it.
//Call this after changing the IP address etc.
begin
acquire.ValueChanged :=  acquire.ValueChanged +1;
acquire.IntfSource := Intf_source;
acquire.TCPAdr := TCP_adr;
acquisition_unit := unit_index;
call_control_driver(c_idle);

if scope_status(CAU_status) <> c_open then 
	begin
	call_control_driver(c_init);
	end;
end;

//**************************************************
procedure scope_close(var status:integer);
{closes cleverscope dll. Returns 0 if all ok, otherwise error code}
begin
if scope_status(CAU_status) <> c_runtime_closed then
	begin
   call_control_driver(c_close);     //closes active unit
	call_control_driver(c_finish);    //closes all units
   end;
status:=  CAU_status;
end;

//*******************************************************
procedure update_values(a_div, b_div, t_div :double ; number_of_points:integer;
         freq,sigvolts,trigvolts:double; trig_chan:integer; trigger_action:word;
         dig_pattern:longword);
begin
acquire.AMaxScale := v_divisions * a_div / 2;
acquire.AMinScale := - acquire.AMaxScale;
acquire.BMaxScale := v_divisions * b_div / 2;
acquire.BMinScale := - acquire.BMaxScale;
acquire.StopTime := t_divisions * t_div /2;
acquire.StartTime := - acquire.StopTime;
acquire.SigGenFreq := freq;
acquire.SigGenAmp := sigvolts;
acquire.TriggerAmplitude := trigvolts;
acquire.TriggerSource := trig_chan;
samples_required := number_of_points;
acquire.AcquireMode := trigger_action;
acquire.ValueChanged:=acquire.ValueChanged+1;
acquire.TriggerSource := trig_chan;
case scope_trigger_channel(trig_chan) of
	trig_chan_a:  acquire.TriggerAmplitude := trigvolts;
	trig_chan_b:  acquire.TriggerAmplitude := trigvolts;
	trig_ext:     acquire.ExtTrigThreshold := trigvolts;
	trig_dig:	  acquire.DigInputThreshold := trigvolts;
   end;
acquire.DigPattern := dig_pattern;
end;

{---------------------------------------------------------}

procedure scope_config (var status:integer);
{configures the acquisition unit. Trig_chan = 0 for Chan A, 1 = chan B
only need to do this if not acquired}

begin

call_control_driver(c_update) ;   //do an update
status:=  CAU_status;
end;

//*************************************************************************

procedure	scope_acquire (var status:integer);
{starts an acquisition. returns error code, = 0 if ok}

begin
call_control_driver(c_acquire);	   //Do an acquire
status:=  CAU_status;
end;

//**************************************************************************
procedure scope_read_waveform (var a_waveform, b_waveform : signal;
                 var num_samples:integer; var delta_t, t_zero : double;
                 var status:integer);
{reads the samples from the buffer, including waveform details}

var	i: integer;

begin
for i:= 0 to samples_returned do
	begin
   a_waveform[i] := a_samples[i];
	b_waveform[i] := b_samples[i];
	end;

delta_t := dt;
t_zero := t0;
num_samples := samples_returned;
status:=  CAU_status;
end;

//**************************************************************************

function check_for_samples:boolean;
         {returns true if we have got the samples}
begin
call_control_driver(c_check);   {Do a check for samples  }
check_for_samples:= (got_samples = 1);
end;

//************************************************************************

function get_CAU_status:integer;
//gets the current CAU status
begin
call_control_driver(c_idle);
get_CAU_status:= CAU_status;
end;

//************************************************************************
//Utility functions:
function string_to_IP(IP_address: string):longword;
	//converts string form of IP address to word.

var	i, c : integer;
		sub:string;
       v, ip_adr:longword;

begin
ip_adr:=0;
while length(IP_address)>0 do
	begin
   i := pos('.',IP_address);
   if i=0 then i := length(IP_address)+1;
   sub := copy(IP_address, 1, i-1);
   val(sub,v,c);
   ip_adr := (ip_adr shl 8) or v;
   delete(IP_address, 1,i);
   end;
string_to_IP :=ip_adr;
end;

//************************************************************

end.
