#using <mscorlib.dll>

using namespace System;
using namespace System::Runtime::InteropServices;

#if _MSC_VER > 1499 // Visual C++ 2008 only
#include <msclr/marshal.h>
using namespace msclr::interop;
#endif

#include "cscope interface.h"
#include "Cscope Control Driver.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>



int32		scope_err;
int32		samples_required;			//the number of samples to capture and display
uInt32		samples_returned;			//actual number of samples returned
float64		dt,t0;						//holds time increment and start value
uInt32		frames_returned;			//number of frames returned after sequential transfer
static	   	TD1	acquire;				//holds the acquire definition
static		TD2	error;					//holds the error value
static		LVBoolean	got_samples;	//set to 1 when we have samples
int32 		frame_number;				//frame number to extract for multi-frame replay
uInt16		CAU_status;					//gives status of CAU. 0 = Runtime Closed, 1 = Closed ,
										//2 = Open, 3 = Fault and open, 4 = Fault and Closed
int32		acquisition_unit;			//Acquisition unit we are talking to, 0..7
unsigned char tx_chars[max_chars];		//contains character array to send to link
unsigned char rx_chars[max_chars];		//contains character array received from link
float64		function_result;			//result of function call

//**************************************************

uInt16 call_cscope_control_driver(uInt16 command)
//use this routine to call the control driver with a particular command
{
CscopeControlDriver(acquisition_unit, command, acquire.StartTime, acquire.StopTime, samples_required,frame_number, &acquire, &got_samples,
                    &t0, &dt, &samples_returned, &frames_returned, a_samples, sizeof(a_samples),  b_samples, sizeof(b_samples),
                    dig_samples, sizeof(dig_samples), &CAU_status, &error);



return CAU_status;
}

//***********************************************************************
float64 get_function(uInt16 cmd_value, float64 cmd_parameter)
//use this routine to call the control driver function command

{
CscopeFunction(acquisition_unit, cmd_value, cmd_parameter,
	tx_chars, sizeof(tx_chars), &function_result, rx_chars, sizeof(rx_chars),
	&error);

return function_result;
}


//**************************************************************
uInt16	scope_init (void)
//Assumes the scope is connected and opens it.
//Sets up the default values

{
acquire.AcquireMode = 1;		//automatic  (0 = single, 1 = automatic, 2 = triggered, 3 = stop)
acquire.AcquisitionMode = 1;	//peak captured
acquire.Acquirer =  4;			//cleverscope is the acquirer
acquire.TransferChans = 2;		//transfer both channels
acquire.AMaxScale = 2;			// Volts range = +/-2
acquire.AMinScale = -2;
acquire.BMaxScale = 2;
acquire.BMinScale = -2;
acquire.AProbe = 0;				//x1
acquire.BProbe = 0;				//x1
acquire.ACoupling = 1;			//DC
acquire.BCoupling = 1;			//DC
acquire.ABandwidth = 1;			//100 MHz
acquire.BBandwidth = 1;			//100 MHz
acquire.TriggerSource = 0;		//A Chan trigger
acquire.TriggerAmplitude = 0;	//Trigger at zero volts
acquire.ATriggerAmplitude = 0;
acquire.BTriggerAmplitude = 0;
acquire.TriggerFilter = 0;		//No trigger filter
acquire.TrigSlope = 0;			//rising
acquire.TriggerHoldoff = 0;
acquire.DigPatternRqd = 0;		//not used
acquire.DigPattern =  0;		//not used
acquire.ExtTrigThreshold = 0;
acquire.DigInputThreshold = 2;
acquire.StartTime =  -0.005;	//-5 msecs
acquire.StopTime =   0.005;		//5 msecs
acquire.PreTrigTime = 0.005;
acquire.Port = 0;
acquire.NumDivisions = 10;
acquire.NumSeqFrames = 1;
acquire.NumBuffers = 2;
acquire.SigGenFreq = 1000;		//1kHz output
acquire.SigGenAmp = 1;			//1V amplitude
acquire.SigGenOffset = 0;
acquire.SigGenWaveform = 0;		//sine
acquire.SigGenSweep = 0;
acquire.SigGenFunc = 0;
acquire.SigGenFreq2 = 0;
acquire.SigGenPhase = 0;
acquire.Trig2Function = 0;		//not used
acquire.MinTriggerPeriod = 0.0000001;
acquire.MaxTriggerPeriod = 1;
acquire.TriggerCount = 1;
acquire.Trig2Slope = 0;
acquire.Trig2SourceChan = 0;
acquire.Trig2Level = 0;
acquire.DigPattern2Rqd = 0;
acquire.DigPattern2 = 0;
acquire.Trigger2Source = 0;
acquire.WaveformAverages = 1;
acquire.ValueChanged = 1;
acquire.SamplerResolution = 0;	//0 = 10 bit (1 = 12 bit, 2 = 14 bit).
samples_required = 1000;
acquire.IntfSource = 0;			//0 = USB, 1 = Ethernet - open given IP address,
								//2 = Ethernet - open next free CAU, 3 = Ethernet - open  given Serial number
acquire.TCPPort = 53270;
acquire.TCPAdr = 0xC0A80169;     //192.168.1.105

acquire.ExtSampleClock = 0;
acquire.UpdateRate =0;
acquire.TransferSize =0;

frame_number = 0;
acquisition_unit = 0;
return call_cscope_control_driver(c_init);
}


//**************************************************
uInt16	cscope_interface(int32 unit_index, uInt16 Intf_source, uInt32 TCP_adr)
//Updates the interface spec. If the CAU_statrus indicates the scope is not open, attempts to open it.
//Call this after changing the IP address etc.
{
acquire.ValueChanged++;
acquire.IntfSource = Intf_source;
acquire.TCPAdr = TCP_adr;
acquisition_unit = unit_index;
CAU_status = call_cscope_control_driver(c_idle);
if (CAU_status != c_open)
	{
	return call_cscope_control_driver(c_init);
	}
else
	{
	return 0;
	}
}

//*************************************************
uInt16 scope_close (void)
//closes the scope
{
call_cscope_control_driver(c_close);
return call_cscope_control_driver(c_finish);

}


//****************************************************
void update_values(float64 a_div, float64 b_div, float64 t_div, 
				    float64 a_div_center, float64 b_div_center, float64 t_div_center, int32 number_of_points,
					float64 freq, float64 sigvolts, float64 trigvolts, int32 trig_chan,
					uInt16 trigger_action, uInt32 dig_pattern)
//updates the acquire variable only. If triggered = 1 then we do triggered capture, otherwise auto capture
{
acquire.AMaxScale = a_div_center + (v_divisions * a_div / 2);
acquire.AMinScale = a_div_center - (v_divisions * a_div / 2);
acquire.BMaxScale = b_div_center + (v_divisions * b_div / 2);
acquire.BMinScale = b_div_center - (v_divisions * b_div / 2);
acquire.StopTime = t_div_center + (t_divisions * t_div /2);
acquire.StartTime = t_div_center - (t_divisions * t_div /2);
acquire.SigGenFreq = freq;
acquire.SigGenAmp = sigvolts;
acquire.TriggerAmplitude = trigvolts;
samples_required = number_of_points;
acquire.AcquireMode = trigger_action; //0= auto, 1 = triggered, 3 = stop
acquire.ValueChanged++;
acquire.TriggerSource = trig_chan;
switch (trig_chan)
    {
    case trig_chan_a:
    	acquire.TriggerAmplitude = trigvolts;
    	break;
    case trig_chan_b:
    	acquire.TriggerAmplitude = trigvolts;
    	break;
   	case trig_ext:
   		acquire.ExtTrigThreshold = trigvolts;
   		break;
   	case trig_dig:
   		acquire.DigInputThreshold = trigvolts;
   		break;
   	}
acquire.DigPattern = dig_pattern;

}

//*******************************************************
uInt16 scope_config (void)
//Configures major values for the acquisition unit.
{
return call_cscope_control_driver(c_update);
}

//*************************************************************************

uInt16	scope_acquire (void)
//start an acquisition

{
return call_cscope_control_driver(c_acquire);
}

//**************************************************************************
uInt16 scope_read_waveform (float32 a_waveform[max_samples], float32 b_waveform[max_samples], int32 *num_samples, float64 *delta_t, float64 *t_zero)
//returns the last read waveform for the given channel
{
uInt32 i;

for (i=0; i<samples_returned; i++)
	{
	a_waveform[i] = a_samples[i];
	}
for (i=0; i<samples_returned; i++)
	{
	b_waveform[i] = b_samples[i];
	}

*delta_t = dt;
*t_zero = t0;
*num_samples = samples_returned;
return CAU_status;
}

//**************************************************************************

uInt16 check_for_samples(void)
//checks to see if samples have been returned. If so returns 1, else 0
{
call_cscope_control_driver(c_check);
return got_samples;
}
//****************************************************************************

uInt16	get_CAU_status(void)
//gets the current CAU status
{
call_cscope_control_driver(c_idle);
return CAU_status;
}

//****************************************************************************

//Support routines



//-------------------------------------------------------------------------
#define PREFIX_START (-24)
/* Smallest power of then for which there is a prefix defined.
   If the set of prefixes will be extended, change this constant
   and update the table "prefix". */

char *eng(float64 value, int digits, int numeric)
{
  static char *prefix[] = {
  "y", "z", "a", "f", "p", "n", "u", "m", "",
  "k", "M", "G", "T", "P", "E", "Z", "Y"
  };
#define PREFIX_END (PREFIX_START+\
(int)((sizeof(prefix)/sizeof(char *)-1)*3))

      int expof10;
      static char result[100];
      char *res = result;

      if (value < 0.)
        {
            *res++ = '-';
            value = -value;
        }
		if (value != 0.0)
			{
			expof10 = (int) log10(value);
		   if(expof10 > 0)
			 expof10 = (expof10/3)*3;
		   else
			 expof10 = (-expof10+3)/3*(-3);

		   value *= pow(10.0,-expof10);

		   if (value >= 1000.)
			  { value /= 1000.0; expof10 += 3; }
		   else if(value >= 100.0)
			  digits -= 2;
		   else if(value >= 10.0)
			  digits -= 1;
		   }
	   else
			{
			expof10 = 0;	//no prefix
			digits=3;
			}
      if(numeric || (expof10 < PREFIX_START) ||
                    (expof10 > PREFIX_END))
        sprintf_s(res,20, "%.*fe%d", digits-1, value, expof10);
      else
        sprintf_s(res,20, "%.*f %s", digits-1, value,
          prefix[(expof10-PREFIX_START)/3]);
      return result;
}

//**********************************************************************************

uInt32 string_to_IP(String ^ ip_string)
//converts the string pointed at by ip_str into an IP address,
//assuming the format nnn.nnn.nnn.nnn
//
{

uInt32	ip_adr;
int8	* pch;
int8	* next_token;
char* ip_str = (char*)Marshal::StringToHGlobalAnsi(ip_string).ToPointer();
ip_adr = 0;
pch = strtok_s( ip_str,".",&next_token);
while (pch != NULL)
	{
	ip_adr = (ip_adr<<8) + atoi(pch);
	pch = strtok_s (NULL, ".",&next_token);
	}
Marshal::FreeHGlobal((IntPtr)ip_str);
return ip_adr;
}
//------------------------------------------------------------------------



