#using <mscorlib.dll>

using namespace System;
using namespace System::Runtime::InteropServices;

#if _MSC_VER > 1499 // Visual C++ 2008 only
#include <msclr/marshal.h>
using namespace msclr::interop;
#endif
#include "stdafx.h"
#include <stdlib.h>
#include <stdio.h>
#include <string>
#include <string.h>
#include <math.h>

#include "cscope interface.h"

//**************************************************************

ref class cscope_interface
{
private:
	static	float	a_samples[max_samples];	  	//contains the a channel samples
	static	float	b_samples[max_samples];	  	//contains the b channel samples
	static	unsigned short	dig_samples[max_samples];	//contains teh digotal channel values

	static    unsigned long scope_err;                   //result of scope error return
	static	_int32		samples_required;			//the number of samples to capture and display
	 static	unsigned long		samples_returned;			//actual number of samples returned
	 static	double		dt,t0;						//holds time increment and start value
	static	unsigned long		frames_returned;			//number of frames returned after sequential transfer
	static  CSCOPE_ACQUIRE	acquire;				//holds the acquire definition
	static	CSCOPE_ERROR    scope_error;					//holds the error value
	static	bool	got_samples;	//set to 1 when we have samples
	static	_int32 		frame_number;				//frame number to extract for multi-frame replay
	static  unsigned short  CAU_status;					//gives status of CAU. 0 = Runtime Closed, 1 = Closed ,
	                                            //2 = Open, 3 = Fault and open, 4 = Fault and Closed
	static  _int32     acquisition_unit;			//Acquisition unit we are talking to, 0..7

public:

//**************************************************

	_int32 call_cscope_control_driver(unsigned short command)
//use this routine to call the control driver with a particular command
{
CscopeControlDriver(command, acquire.StartTime, acquire.StopTime, samples_required,frame_number, &acquire, &got_samples,
                    &t0, &dt, &samples_returned, &frames_returned, a_samples, max_samples*4,  b_samples, max_samples*4,
                    dig_samples, max_samples*2, &scope_error);
                    
scope_err = error.code;                    
return scope_err;
}

//******************************************************

unsigned short	scope_init()
//Assumes the scope is connected and opens it.
//Sets up the default values

{
String^ str = "test";

acquire.AcquireMode = 1;		//automatic  (0 = single, 1 = automatic, 2 = triggered, 3 = stop)
acquire.AcquisitionMode = 1;	//peak captured
acquire.Acquirer =  4;			//cleverscope is the acquirer
acquire.TransferChans = 2;		//transfer both channels
acquire.AMaxScale = 2;			// Volts range = +/-2
acquire.AMinScale = -2;
acquire.BMaxScale = 2;
acquire.BMinScale = -2;
acquire.AProbe = 0;				//x1
acquire.BProbe = 0;				//x1
acquire.ACoupling = 1;			//DC
acquire.BCoupling = 1;			//DC
acquire.ABandwidth = 1;			//100 MHz
acquire.BBandwidth = 1;			//100 MHz
acquire.TriggerSource = 0;		//A Chan trigger
acquire.TriggerAmplitude = 0;	//Trigger at zero volts
acquire.ATriggerAmplitude = 0;
acquire.BTriggerAmplitude = 0;
acquire.TriggerFilter = 0;		//No trigger filter
acquire.TrigSlope = 0;			//rising
acquire.TriggerHoldoff = 0;
acquire.DigPatternRqd = 0;		//not used
acquire.DigPattern =  0;		//not used
acquire.ExtTrigThreshold = 0;
acquire.DigInputThreshold = 2;
acquire.StartTime =  -0.005;	//-5 msecs
acquire.StopTime =   0.005;		//5 msecs
acquire.PreTrigTime = 0.005;
acquire.Port = 0;
acquire.NumDivisions = 10;
acquire.NumSeqFrames = 1;
acquire.NumBuffers = 2;
acquire.SigGenFreq = 1000;		//1kHz output
acquire.SigGenAmp = 1;			//1V amplitude
acquire.SigGenOffset = 0;
acquire.SigGenWaveform = 0;		//sine
acquire.SigGenSweep = 0;
acquire.SigGenFunc = 0;
acquire.SigGenFreq2 = 0;
acquire.SigGenPhase = 0;
acquire.Trig2Function = 0;		//not used
acquire.MinTriggerPeriod = 0.0000001;
acquire.MaxTriggerPeriod = 1;
acquire.TriggerCount = 1;
acquire.Trig2Slope = 0;
acquire.Trig2SourceChan = 0;
acquire.Trig2Level = 0;
acquire.DigPattern2Rqd = 0;
acquire.DigPattern2 = 0;
acquire.Trigger2Source = 0;
acquire.WaveformAverages = 1;
acquire.ValueChanged = 1;
acquire.SamplerResolution = 0;	//0 = 10 bit (1 = 12 bit, 2 = 14 bit).
samples_required = 1000;
acquire.IntfSource = 0;			//0 = USB, 1 = Ethernet - open given IP address,
//2 = Ethernet - open next free CAU, 3 = Ethernet - open  given Serial number
acquire.TCPPort = 53270;
acquire.TCPAdr = 0xC0A80168;
acquire.CAUSerNumHi = 0;
acquire.CAUSerNumLo = 0;
acquire.UnitsAreLinked = 0;
acquire.ExtSampleClock = 0;
acquire.UpdateRate = 0;
acquire.TransferSize = 0;

frame_number = 0;
acquisition_unit = 0;
return call_cscope_control_driver(c_init);
}

//**************************************************
unsigned short cscope_interface(_int32 unit_index, unsigned short Intf_source, unsigned long TCP_adr)
//Updates the interface spec. If the CAU_status indicates the scope is not open, attempts to open it.
//Call this after changing the IP address etc.
{
    acquire.ValueChanged++;
    acquire.IntfSource = Intf_source;
    acquire.TCPAdr = TCP_adr;
    acquisition_unit = unit_index;
    CAU_status = call_cscope_control_driver(c_idle);
    if (CAU_status != c_open)
    {
        return call_cscope_control_driver(c_init);
    }
    else
    {
        return 0;
    }
}


//*************************************************
unsigned short scope_close()
//closes the scope
{
    call_cscope_control_driver(c_close);
    return call_cscope_control_driver(c_finish);
}


//****************************************************
void update_values(double a_div, double b_div, double time_div, _int32 number_of_points,
					double freq, double sigvolts, double trigvolts, unsigned long trig_chan, unsigned short trigger_action, unsigned long dig_pattern)
//updates the acquire variable only. If triggered = 1 then we do triggered capture, otherwise auto capture
{
acquire.AMaxScale = v_divisions * a_div / 2;
acquire.AMinScale = - acquire.AMaxScale;
acquire.BMaxScale = v_divisions * b_div / 2;
acquire.BMinScale = - acquire.BMaxScale;
acquire.StopTime = t_divisions * time_div /2;
acquire.StartTime = - acquire.StopTime;
acquire.SigGenFreq = freq;
acquire.SigGenAmp = sigvolts;
acquire.TriggerAmplitude = trigvolts;
acquire.TriggerSource = trig_chan;
samples_required = number_of_points;
acquire.AcquireMode = trigger_action; //0= auto, 1 = triggered, 3 = stop
acquire.ValueChanged++;
switch (trig_chan)
{
    case trig_chan_a:
        acquire.TriggerAmplitude = trigvolts;
        break;
    case trig_chan_b:
        acquire.TriggerAmplitude = trigvolts;
        break;
    case trig_ext:
        acquire.ExtTrigThreshold = trigvolts;
        break;
    case trig_dig:
        acquire.DigInputThreshold = trigvolts;
        break;
}
acquire.DigPattern = dig_pattern;

}

//*******************************************************
unsigned short scope_config()
//Configures major values for the acquisition unit.
{
return call_cscope_control_driver(c_update);
}

//*************************************************************************

unsigned short scope_acquire()
//start an acquisition

{
return call_cscope_control_driver(c_acquire);
}

//**************************************************************************
unsigned short scope_read_waveform(float a_waveform[max_samples], float b_waveform[max_samples], _int32 *num_samples, double *delta_t, double *t_zero)
//returns the last read waveform for the given channel
{
unsigned long i;

for (i=0; i<samples_returned; i++)
	{
	a_waveform[i] = a_samples[i];
	}
for (i=0; i<samples_returned; i++)
	{
	b_waveform[i] = b_samples[i];
	}

delta_t = dt;
t_zero = t0;
num_samples = samples_returned;
return CAU_status;
}

//**************************************************************************

bool check_for_samples()
//checks to see if samples have been returned. If so returns 1, else 0
{
call_cscope_control_driver(c_check);
return (got_samples !=0);
}

    //****************************************************************************

unsigned short	get_CAU_status()
//gets the current CAU status
{
call_cscope_control_driver(c_idle);
return CAU_status;
}

int32	get_function(uInt16 cmd_value)
//gets the function value
{
frame_number = cmd_value;
call_cscope_control_driver(c_function);
return frames_returned;
}


//****************************************************************************

//Support routines



//------------------------------------------------------------------------
//-------------------------------------------------------------------------

/* Smallest power of then for which there is a prefix defined.
   If the set of prefixes will be extended, change this constant
   and update the table "prefix". */

char * eng(double value, int digits, int numeric)
{
	const	short PREFIX_START= -24;
	const	short PREFIX_END= 24;
  static char *prefix[] = {
  "y", "z", "a", "f", "p", "n", "u", "m", "",
  "k", "M", "G", "T", "P", "E", "Z", "Y"
  };
#define PREFIX_END (PREFIX_START+\
(int)((sizeof(prefix)/sizeof(char *)-1)*3))

      int expof10;
      static char result[100];
      char *res = result;

      if (value < 0.)
        {
            *res++ = '-';
            value = -value;
        }
		if (value != 0.0)
			{
			expof10 = (int) log10(value);
		   if(expof10 > 0)
			 expof10 = (expof10/3)*3;
		   else
			 expof10 = (-expof10+3)/3*(-3);

		   value *= pow(10.0,-expof10);

		   if (value >= 1000.)
			  { value /= 1000.0; expof10 += 3; }
		   else if(value >= 100.0)
			  digits -= 2;
		   else if(value >= 10.0)
			  digits -= 1;
		   }
	   else
			{
			expof10 = 0;	//no prefix
			digits=3;
			}
      if(numeric || (expof10 < PREFIX_START) ||
                    (expof10 > PREFIX_END))
        sprintf_s(res,20, "%.*fe%d", digits-1, value, expof10);
      else
        sprintf_s(res,20, "%.*f %s", digits-1, value,
          prefix[(expof10-PREFIX_START)/3]);
      return result;
}

//------------------------------------------------------------------------
unsigned _int32 string_to_IP(String ^ ip_string)
//converts the string pointed at by ip_str into an IP address,
//assuming the format nnn.nnn.nnn.nnn
//
{


unsigned _int32	ip_adr;
_int8	* pch;
_int8	* next_token;
char* ip_str = (char*)Marshal::StringToHGlobalAnsi(ip_string).ToPointer();
ip_adr = 0;
pch = strtok_s( ip_str,".",&next_token);
while (pch != NULL)
	{
	ip_adr = (ip_adr<<8) + atoi(pch);
	pch = strtok_s (NULL, ".",&next_token);
	}
Marshal::FreeHGlobal((IntPtr)ip_str);
return ip_adr;
}

//------------------------------------------------------------------------
}; //class