'============================================================================
'Cleverscope Simple Scope
'The code contained in this file (only) is released into the public domain, so you
'can copy it into your project without any license encumbrance.  
'
'This file is distributed in the hope that it will be useful,
'but WITHOUT ANY WARRANTY without even the implied warranty of
'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
'=============================================================================

Imports ZedGraph
Imports System.Threading
Imports SimpleScope

'****************************************************************************

Public Class SimpleForm

    Enum e_calibration As UShort
        Idle
        StartCalibrate
        GndOfsAmeasure
        GndOfsBMeasure
        BaselineMeasure
        RefAMeasure
        RefBMeasure
        SaveHWValues
        StartSigGenCal
        StartExtTrigCal
        StartDigInpCal
        DoCalibrate
        SetRef
    End Enum

    Enum e_functions As UShort
        GetSerialNum
        GetFirmwareVer
        GetDriverVer
        GetResolution
        GetFrameLength
        GetTemperature
        StartLinkSend
        SendLinkData
        ReadLinkData
        ActiveMessage
        Calibrate
        SetCalRef
        SamplingStatus
    End Enum


    Dim cscope As New cscope_intf

    Const num_points As Integer = 2000

    Dim a_vert As Double, b_vert As Double, horz As Double
    Dim sig_gen_volts As Double, sig_gen_freq As Double
    Dim trig_channel As UShort

    Dim trig_volts As Double

    Dim single_acquire As Boolean
    Dim auto_acquire As Boolean
    Dim a_waveform(cscope_intf.max_samples) As Double
    Dim b_waveform(cscope_intf.max_samples) As Double


    Dim waiting_for_trigger As Boolean  '0 means we have not started an acquire. 1 means we are waiting for the trigger
    Dim trigger_action As UShort    'for how to capture, 0 = auto, 1 = triggered, 3 = stop
    Dim vals_changed As Boolean

    Dim digital_pattern As UInteger 'digital pattern used for digital triggering

    Dim Interface_Source As UShort      'The interface source
    Dim Interface_address As UInteger   'the IP address for the interface
    Dim CAU_status As UShort            '{c_runtime_closed, c_closed, c_open,  c_fault, c_fault_closed, c_open_pending}
    Dim Cscope_index As UInteger            'index into current cleverscope
    Dim function_value As Integer   'gives the function number
    Dim f_timer_active As Boolean    'true if timer may become active
    Dim stop_sync As Integer        'used for synchronizing stop with timer.
    Dim f_calibrating As Boolean        'true if we are calibrating
    Dim calibration As e_calibration    'holds current calibration value
    Dim functions As e_functions    'holds current functions value


    Private Sub Setval(ByVal ctrl As System.Windows.Forms.Label, _
         ByVal value As Double, ByVal suffix As String)

        ctrl.Text = cscope.eng(value, 3) + suffix
    End Sub

    Private Sub SetvalInt(ByVal ctrl As System.Windows.Forms.Label, _
     ByVal value As Double, ByVal suffix As String)

        ctrl.Text = value.ToString + suffix
    End Sub


    Private Sub InitializeGUI()

        a_vert = 1.0
        b_vert = 1.0
        horz = 0.001
        sig_gen_volts = 1.0
        sig_gen_freq = 1000.0
        trig_channel = 0
        trig_volts = 0.0
        single_acquire = False
        auto_acquire = False
        waiting_for_trigger = False
        digital_pattern = &H10001
        Interface_Source = 0
        Interface_address = 3232235880  '&HC0A80168
        function_value = 0
        CAU_status = 0
        f_timer_active = False
        stop_sync = 0

        single_acquire = False
        auto_acquire = False
        waiting_for_trigger = False
        vals_changed = True
        f_calibrating = False
        calibration = e_calibration.Idle
        functions = e_functions.GetSerialNum

        Setval(Adiv, a_vert, "V")

        Setval(Bdiv, b_vert, "V")
        Setval(Tdiv, horz, "s")
        Setval(TrigV, trig_volts, "V")
        Setval(SigGenF, sig_gen_freq, "Hz")
        Setval(SigGenV, sig_gen_volts, "V")


        SetA_Vertical.Value = 6
        SetB_Vertical.Value = 6
        SetHorizontal.Value = 9
        Set_SigGen_Freq.Value = 9
        Set_SigGen_Volts.Value = 5

        zedGraphControl1.GraphPane.YAxis.Scale.Min = -a_vert * cscope_intf.v_divisions / 2
        zedGraphControl1.GraphPane.YAxis.Scale.Max = a_vert * cscope_intf.v_divisions / 2
        zedGraphControl1.GraphPane.YAxis.Scale.MajorStep = a_vert
        zedGraphControl1.GraphPane.Y2Axis.Scale.Min = -b_vert * cscope_intf.v_divisions / 2
        zedGraphControl1.GraphPane.Y2Axis.Scale.Max = b_vert * cscope_intf.v_divisions / 2
        zedGraphControl1.GraphPane.Y2Axis.Scale.MajorStep = b_vert
        zedGraphControl1.GraphPane.XAxis.Scale.Min = -horz * cscope_intf.v_divisions / 2
        zedGraphControl1.GraphPane.XAxis.Scale.Max = horz * cscope_intf.v_divisions / 2
        zedGraphControl1.GraphPane.XAxis.Scale.MajorStep = horz
        stop_sync = 0
        f_timer_active = True
        Timer1.Enabled = True
    End Sub

    ' Build the Chart
    Private Sub CreateGraph(ByVal zgc As ZedGraphControl)

        Dim myPane As GraphPane = zedGraphControl1.GraphPane

        ' Set the titles and axis labels
        myPane.Title.Text = "Simple Scope"
        myPane.XAxis.Title.Text = "Time (s)"
        myPane.YAxis.Title.Text = "A Volts"
        myPane.Y2Axis.Title.Text = "B Volts"

        ' pur a couple of points on the graph
        Dim list1 As New PointPairList
        Dim list2 As New PointPairList
        Dim i As Integer, x As Double, y1 As Double, y2 As Double

        For i = 0 To 1
            x = 0
            y1 = 0
            y2 = 0
            list1.Add(x, y1)
            list2.Add(x, y2)
        Next i

        ' Generate a red curve 
        Dim myCurve As LineItem
        myCurve = myPane.AddCurve("", list1, Color.Red, SymbolType.None)
        ' Generate a blue curve with circle symbols, and "Beta" in the legend
        myCurve = myPane.AddCurve("", list2, Color.Blue, SymbolType.None)
        ' Fill the symbols with white
        myCurve.IsY2Axis = True

        ' Show the x axis grid
        myPane.XAxis.MajorTic.IsOpposite = False
        myPane.XAxis.MinorTic.IsOpposite = False
        myPane.XAxis.MajorGrid.IsVisible = True

        ' Make the Y axis scale red
        myPane.YAxis.Scale.FontSpec.FontColor = Color.Red
        myPane.YAxis.Title.FontSpec.FontColor = Color.Red
        ' turn off the opposite tics so the Y tics don't show up on the Y2 axis
        myPane.YAxis.MajorTic.IsOpposite = False
        myPane.YAxis.MinorTic.IsOpposite = False
        ' Don't display the Y zero line
        myPane.YAxis.MajorGrid.IsZeroLine = False
        myPane.YAxis.MajorGrid.IsVisible = True
        ' Align the Y axis labels so they are flush to the axis
        myPane.YAxis.Scale.Align = AlignP.Inside
        ' Manually set the axis range
        myPane.YAxis.Scale.Min = -10
        myPane.YAxis.Scale.Max = 10

        ' Enable the Y2 axis display
        myPane.Y2Axis.IsVisible = True
        ' Make the Y2 axis scale blue
        myPane.Y2Axis.Scale.FontSpec.FontColor = Color.Blue
        myPane.Y2Axis.Title.FontSpec.FontColor = Color.Blue
        ' turn off the opposite tics so the Y2 tics don't show up on the Y axis
        myPane.Y2Axis.MajorTic.IsOpposite = False
        myPane.Y2Axis.MinorTic.IsOpposite = False
        ' Don't Display the Y2 axis grid lines
        myPane.Y2Axis.MajorGrid.IsVisible = False
        ' Align the Y2 axis labels so they are flush to the axis
        myPane.Y2Axis.Scale.Align = AlignP.Inside
        myPane.Y2Axis.MajorGrid.IsZeroLine = False

        ' Tell ZedGraph to calculate the axis ranges
        ' Note that you MUST call this after enabling IsAutoScrollRange, since AxisChange() sets
        ' up the proper scrolling parameters
        zedGraphControl1.AxisChange()
        ' Make sure the Graph gets redrawn
        zedGraphControl1.Invalidate()

    End Sub


    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Dim scope_err As UShort
        CreateGraph(zedGraphControl1)
        scope_err = cscope.scope_init()
        InitializeGUI()
    End Sub


    Private Sub SetA_Vertical_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SetA_Vertical.ValueChanged
        a_vert = cscope.v_div(SetA_Vertical.Value)
        Setval(Adiv, a_vert, "V")
        vals_changed = True
        zedGraphControl1.GraphPane.YAxis.Scale.Min = -a_vert * cscope_intf.v_divisions / 2
        zedGraphControl1.GraphPane.YAxis.Scale.Max = a_vert * cscope_intf.v_divisions / 2
        zedGraphControl1.GraphPane.YAxis.Scale.MajorStep = a_vert
        zedGraphControl1.GraphPane.AxisChange()
        zedGraphControl1.Invalidate()

    End Sub

    Private Sub SetB_Vertical_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SetB_Vertical.ValueChanged
        b_vert = cscope.v_div(SetB_Vertical.Value)
        Setval(Bdiv, b_vert, "V")
        vals_changed = True
        zedGraphControl1.GraphPane.Y2Axis.Scale.Min = -b_vert * cscope_intf.v_divisions / 2
        zedGraphControl1.GraphPane.Y2Axis.Scale.Max = b_vert * cscope_intf.v_divisions / 2
        zedGraphControl1.GraphPane.Y2Axis.Scale.MajorStep = b_vert
        zedGraphControl1.GraphPane.AxisChange()
        zedGraphControl1.Invalidate()

    End Sub

    Private Sub SetHorizontal_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SetHorizontal.ValueChanged
        horz = cscope.t_div(SetHorizontal.Value)
        Setval(Tdiv, horz, "sec")
        vals_changed = True
        zedGraphControl1.GraphPane.XAxis.Scale.Min = -horz * cscope_intf.t_divisions / 2
        zedGraphControl1.GraphPane.XAxis.Scale.Max = horz * cscope_intf.t_divisions / 2
        zedGraphControl1.GraphPane.XAxis.Scale.MajorStep = horz
        zedGraphControl1.GraphPane.AxisChange()
        zedGraphControl1.Invalidate()

    End Sub

    Private Sub Set_SigGen_Freq_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Set_SigGen_Freq.ValueChanged
        sig_gen_freq = cscope.freqs(Set_SigGen_Freq.Value)
        Setval(SigGenF, sig_gen_freq, "Hz")
        vals_changed = True

    End Sub

    Private Sub Set_SigGen_Volts_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Set_SigGen_Volts.ValueChanged
        sig_gen_volts = Set_SigGen_Volts.Value * 0.2
        Setval(SigGenV, sig_gen_volts, "V")
        vals_changed = True

    End Sub

    Private Sub Set_Trigger_Volts_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Set_Trigger_Volts.ValueChanged
        trig_volts = Set_Trigger_Volts.Value * 0.2
        Setval(TrigV, trig_volts, "V")
        vals_changed = True

    End Sub

    Private Sub SingleAcquire_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SingleAcquire.Click
        vals_changed = True
        Trig_LED.Visible = False
        auto_acquire = False
        waiting_for_trigger = False
        TriggeredAcquire.Text = "Auto"
        If (single_acquire) Then

            single_acquire = False
            SingleAcquire.Text = "Single"
            trigger_action = cscope_intf.acq_stop

            'stop doing stuff

        Else
            single_acquire = True
            SingleAcquire.Text = "Wait.."
            trigger_action = cscope_intf.acq_single   'use triggered captured
        End If

    End Sub

    Private Sub TriggeredAcquire_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TriggeredAcquire.Click
        vals_changed = True
        single_acquire = False
        Trig_LED.Visible = False
        waiting_for_trigger = False
        SingleAcquire.Text = "Single"
        If (auto_acquire = True) Then
            auto_acquire = False
            TriggeredAcquire.Text = "Auto"
            trigger_action = cscope_intf.acq_stop   'stop doing stuff
        Else
            auto_acquire = True
            TriggeredAcquire.Text = "Triggering..."
            trigger_action = cscope_intf.acq_auto   'use auto captured
        End If

    End Sub

    Private Sub Timer1_Tick_1(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick
        'Handles Timer1.Tick
        Dim i As Integer
        Dim xval As Double
        Dim num_samples As UInteger
        Dim dT As Double
        Dim T0 As Double
        Dim a_waveform(cscope_intf.max_samples) As Single
        Dim b_waveform(cscope_intf.max_samples) As Single
        Dim status As UShort
        Dim sync As Integer
        Dim ip1 As IPointListEdit
        Dim ip2 As IPointListEdit
        Dim cal_result As Double

        num_samples = 0
        dT = 0
        T0 = 0
        sync = Interlocked.CompareExchange(stop_sync, 1, 0)        'take sync to stop close

        If (sync = 0) Then   'not being used by close window

            Timer1.Enabled = False          'we stop timer events, in case this process takes longer
            'than a timer tick
            status = cscope.get_CAU_status()

            If (CAU_status <> status) Then
                CAU_status = status
                Cscope_Status.SelectedIndex = CAU_status
            End If
            If (vals_changed) Then   'here we put the latest set of updates in the acquisition array
                'these will be actioned when we do an acquire, or down below if there is no acquire.
                cscope.update_values(a_vert, b_vert, horz, num_points, _
                    sig_gen_freq, sig_gen_volts, trig_volts, trig_channel, trigger_action, digital_pattern)
            End If
            If (Not (waiting_for_trigger) And (single_acquire Or auto_acquire)) Then
                Trig_LED.Visible = False
                waiting_for_trigger = True
                vals_changed = False        'will have been updated by acquire. We do it here so that if vals_changed
                cscope.scope_acquire()  'is set true while acquiring, there
            End If
            If (waiting_for_trigger And cscope.check_for_samples()) Then
                Trig_LED.Visible = True
                waiting_for_trigger = False
                If (single_acquire) Then
                    SingleAcquire.Text = "Single"
                End If
                single_acquire = False
                cscope.scope_read_waveform(a_waveform, b_waveform, num_samples, dT, T0)
                xval = T0
                ip1 = zedGraphControl1.GraphPane.CurveList(0).Points
                ip2 = zedGraphControl1.GraphPane.CurveList(1).Points

                If (Not IsNothing(ip1) And (Not IsNothing(ip2)) And (num_samples > 0)) Then
                    ip1.Clear()
                    ip2.Clear()
                    For i = 0 To num_samples - 1
                        ip1.Add(xval, a_waveform(i))
                        ip2.Add(xval, b_waveform(i))
                        xval = xval + dT
                    Next
                End If
                zedGraphControl1.Invalidate()
            End If

            If (f_calibrating) Then       'we want to run the calibration system

                cal_result = cscope.get_function(e_functions.Calibrate, e_calibration.DoCalibrate)
                SetvalInt(Cmd_value, cal_result, " ")
                If ((cal_result < 0) Or (cal_result = 100)) Then
                    f_calibrating = False
                    calibration = e_calibration.Idle
                    CalibrateStep.SelectedIndex = e_calibration.Idle
                End If
            End If

            If (vals_changed) Then
                'vals have changed that couldn't be handled by the acquire.
                cscope.update_values(a_vert, b_vert, horz, num_points, _
                            sig_gen_freq, sig_gen_volts, trig_volts, trig_channel, trigger_action, digital_pattern)
                cscope.scope_config()
                vals_changed = False        'updated if acquire did not update them
            End If


            Timer1.Enabled = f_timer_active 'go back to allowing timer events unless closing
            stop_sync = 0       'release sync
            If (Not f_timer_active) Then            'we do it here at a nice quantized point
                cscope.scope_close()
                Me.Close()
            End If
        Else
            'we are in the middle of closing down
        End If

    End Sub

    Private Sub QuitButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles QuitButton.Click
        f_timer_active = False
        '           timer1.Enabled = false
        '           cscope.scope_close()
        '            this.Close()
    End Sub

    Private Sub IP_address_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles IP_address.TextChanged
        Interface_address = cscope.string_to_IP(IP_address.Text)
        cscope.cscope_interface(Cscope_index, Interface_Source, Interface_address)

    End Sub

    Private Sub Cscope_unit_index_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Cscope_unit_index.SelectedIndexChanged
        Cscope_index = Cscope_unit_index.SelectedIndex
        cscope.cscope_interface(Cscope_index, Interface_Source, Interface_address)


    End Sub

    Private Sub Intf_source_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Intf_source.SelectedIndexChanged
        Interface_Source = Intf_source.SelectedIndex
        cscope.cscope_interface(Cscope_index, Interface_Source, Interface_address)
    End Sub

    Private Sub Restart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Restart.Click
        cscope.cscope_interface(Cscope_index, Interface_Source, Interface_address)

    End Sub

    Private Sub Trigger_Channel_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Trigger_Channel.SelectedIndexChanged
        trig_channel = Trigger_Channel.SelectedIndex
        vals_changed = True

    End Sub

    Private Sub Cmd_get_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Cmd_get.Click
        SetvalInt(Cmd_value, cscope.get_function(functions, calibration), " ")
        If (functions = e_functions.Calibrate) Then
            If (Not ((calibration = e_calibration.Idle) Or (calibration = e_calibration.SaveHWValues) Or (calibration = e_calibration.SetRef))) Then
                f_calibrating = True
                calibration = e_calibration.DoCalibrate
                CalibrateStep.SelectedIndex = e_calibration.DoCalibrate
            End If
        End If
    End Sub

    Private Sub Cmd_Function_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Cmd_Function.SelectedIndexChanged
        functions = Cmd_Function.SelectedIndex
    End Sub

    Private Sub CalibrateStep_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CalibrateStep.SelectedIndexChanged
        calibration = CalibrateStep.SelectedIndex
    End Sub
End Class
